==> hermes-2.4.10.4.tar.gz: hermes-2.4.10.4/app/models/hermes/nud.rb <==
# frozen_string_literal: true

module Hermes
  # Nud class
  class Nud
    # Linked to the method of housing_request :
    #   - nud_creations
    include Hermes::Concerns::CommonFieldsConcern
    include Hermes::Concerns::ActivableConcern
    include Hermes::Concerns::SopraXmlSender
    
    field :applicant_id, type: BSON::ObjectId
    field :department, type: String
    field :creation_date, type: DateTime
    field :unique_number, type: String
    field :provider, type: String
    
    writer_auth_roles 'collaborateur entreprise', "collaborateur offre logement", 'superadministrateur'
    
    belongs_to(
      :housing_request,
      class_name: 'Hermes::HousingRequest',
      index: true, touch: true
    )
    
    belongs_to(
      :applicant,
      class_name: 'Hermes::Applicant',
      index: true, touch: true
    )
    validates_presence_of :housing_request
    
    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id
    
    common_indexes *%i[
      department
      creation_date
      unique_number
      is_active
      provider
    ]
    
    before_save :before_save_actions
    before_destroy :before_destroy_actions
    #after_save      :after_save_actions
    after_create :after_create_actions
    after_destroy :after_destroy_actions
    
    ###############################
    #### Notification #############
    include ::Concerns::NotificationCenter::Notifiable
    
    def self.targets_selector(nud, opt = {})
      [nud.applicant]
    end
    
    def notifiable_title(key, opt = {})
      I18n.t("#{key}.title")
    end
    
    def notifiable_body(key, opt = {})
      I18n.t("#{key}.body", nud: unique_number)
    end
    
    def notifiable_path
      "https://als-hermes-int.be-idys.com/nuds/#{unique_number}"
    end
    
    def notification_mailer
      ::Hermes::NotificationCenter::HermesMailer
    end
    
    def update_provider
      if HERMES_CONFIG["DEP_PROVIDERS"]
        HERMES_CONFIG["DEP_PROVIDERS"].each do |key, val|
          val.each do |dep|
            if (unique_number[0..2] === (dep.length == 2 ? "0" + dep : dep))
              self.provider = key
            end
          end
        end
      end
    end
    
    def update_all_nud_providers
      nuds = Hermes::Nud.where(provider: nil)
      nuds.length
      
      nuds.each_with_index do |nud, i|
        if nud.unique_number
          nud.update_provider
          nud.save
          
          ap i
        else
          ap nud
        end
      end
    end
    
    def update_housing_request_unique_numbers
      if housing_request
        unique_numbers = []
        housing_request.nuds.each do |nud|
          unique_numbers.push(nud.unique_number)
        end
        housing_request.unique_numbers = unique_numbers
      end
    end
    
    def script_reprise_update_housing_request_unique_numbers
      nuds = Hermes::Nud.where({})
      nuds.length
      nuds.batch_size(5).each_with_index do |nud, i|
        nud.update_housing_request_unique_numbers
        nud.save
        ap i
      end
    end
    
    ###############################
    ###############################
    
    # ========================== #
    # - SIDE EFFETCTS HANDLERS - #
    # ========================== #
    def before_save_actions
      update_provider if unique_number_changed?
      update_applicant_id if housing_request_id_changed?
    end
    
    def before_destroy_actions
      housing_request.nud_ids.delete(id)
      housing_request.save
    end
    
    def after_create_actions
      housing_request.save
      update_affiliated_scores if oldest_for_department?
      notify(key: NOTIFICATION_KEYS[:nud][:create], channels: ['email'])
      update_housing_request_unique_numbers
    end
    
    def after_update_actions
      housing_request.save unless applicant_id_changed? # to avoid infinit loop
      update_affiliated_scores if must_update_affiliated_scores?
    end
    
    def after_destroy_actions
      update_affiliated_scores if oldest_for_department?
      update_housing_request_unique_numbers
    end
    
    # ============================== #
    # - SIDE EFFETCTS DEPENDENCIES - #
    # ============================== #
    def update_applicant_id
      self.applicant_id = housing_request.main_applicant_id
    end
    
    def oldest_for_department?
      # has the oldest @creation_date
      # WITHIN @demartement.nuds
      # WHERE_SAME @housing_request
      oldest_other = housing_request.nuds
                       .where(
                         id: {:$ne => id},
                         department: department,
                         creation_date: {:$lte => creation_date}
                       ).order_by(creation_date: 1)
                       &.first
                       &.creation_date
      oldest_other.blank? || oldest_other > creation_date
    end
    
    # Scores update trigger
    def must_update_affiliated_scores?
      # IF
      #   any of fields list has changed
      # AND
      %w[
        creation_date
        department
        applicant_id
        is_active
      ].any? {|field_name| send("#{field_name}_changed?")} &&
        oldest_for_department?
    end
    
    # Affiliated scores are all that match
    #   SAME @department && @applicant_id
    def update_affiliated_scores
      ToucheScoresJob.perform_later(
        Hermes::Score.only(:id, :department, :applicant_id)
          .where(
            department: department,
            applicant_id: housing_request.main_applicant_id
          ).pluck(:id)
      )
    end
  end
end

==> hermes-2.4.11.4.tar.gz: hermes-2.4.11.4/app/models/hermes/nud.rb <==
# frozen_string_literal: true

module Hermes
  # Nud class
  class Nud
    # Linked to the method of housing_request :
    #   - nud_creations
    include Hermes::Concerns::CommonFieldsConcern
    include Hermes::Concerns::ActivableConcern
    include Hermes::Concerns::SopraXmlSender
    
    field :applicant_id, type: BSON::ObjectId
    field :department, type: String
    field :creation_date, type: DateTime
    field :unique_number, type: String
    field :provider, type: String
    
    writer_auth_roles 'collaborateur entreprise', "collaborateur offre logement", 'superadministrateur'
    
    belongs_to(
      :housing_request,
      class_name: 'Hermes::HousingRequest',
      index: true, touch: true
    )
    
    belongs_to(
      :applicant,
      class_name: 'Hermes::Applicant',
      index: true, touch: true
    )
    validates_presence_of :housing_request
    
    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id
    
    common_indexes *%i[
      department
      creation_date
      unique_number
      is_active
      provider
    ]
    
    before_save :before_save_actions
    before_destroy :before_destroy_actions
    #after_save      :after_save_actions
    after_create :after_create_actions
    after_destroy :after_destroy_actions
    
    ###############################
    #### Notification #############
    include ::Concerns::NotificationCenter::Notifiable
    
    def self.targets_selector(nud, opt = {})
      [nud.applicant]
    end
    
    def notifiable_title(key, opt = {})
      I18n.t("#{key}.title")
    end
    
    def notifiable_body(key, opt = {})
      I18n.t("#{key}.body", nud: unique_number)
    end
    
    def notifiable_path
      "https://als-hermes-int.be-idys.com/nuds/#{unique_number}"
    end
    
    def notification_mailer
      ::Hermes::NotificationCenter::HermesMailer
    end
    
    def update_provider
      if HERMES_CONFIG["DEP_PROVIDERS"]
        HERMES_CONFIG["DEP_PROVIDERS"].each do |key, val|
          val.each do |dep|
            if (unique_number[0..2] === (dep.length == 2 ? "0" + dep : dep))
              self.provider = key
            end
          end
        end
      end
    end
    
    def update_all_nud_providers
      nuds = Hermes::Nud.where(provider: nil)
      nuds.length
      
      nuds.each_with_index do |nud, i|
        if nud.unique_number
          nud.update_provider
          nud.save
          
          ap i
        else
          ap nud
        end
      end
    end
    
    def update_housing_request_unique_numbers
      if housing_request
        unique_numbers = []
        housing_request.nuds.each do |nud|
          unique_numbers.push(nud.unique_number)
        end
        housing_request.update(unique_numbers: unique_numbers)
      end
    end
    
    def script_reprise_update_housing_request_unique_numbers
      nuds = Hermes::Nud.where({})
      nuds.length
      nuds.batch_size(5).each_with_index do |nud, i|
        nud.update_housing_request_unique_numbers
        nud.save
        ap i
      end
    end
    
    ###############################
    ###############################
    
    # ========================== #
    # - SIDE EFFETCTS HANDLERS - #
    # ========================== #
    def before_save_actions
      update_provider if unique_number_changed?
      update_applicant_id if housing_request_id_changed?
    end
    
    def before_destroy_actions
      housing_request.nud_ids.delete(id)
      housing_request.save
    end
    
    def after_create_actions
      housing_request.save
      update_affiliated_scores if oldest_for_department?
      notify(key: NOTIFICATION_KEYS[:nud][:create], channels: ['email'])
      update_housing_request_unique_numbers
    end
    
    def after_update_actions
      housing_request.save unless applicant_id_changed? # to avoid infinit loop
      update_affiliated_scores if must_update_affiliated_scores?
    end
    
    def after_destroy_actions
      update_affiliated_scores if oldest_for_department?
      update_housing_request_unique_numbers
    end
    
    # ============================== #
    # - SIDE EFFETCTS DEPENDENCIES - #
    # ============================== #
    def update_applicant_id
      self.applicant_id = housing_request.main_applicant_id
    end
    
    def oldest_for_department?
      # has the oldest @creation_date
      # WITHIN @demartement.nuds
      # WHERE_SAME @housing_request
      oldest_other = housing_request.nuds
                       .where(
                         id: {:$ne => id},
                         department: department,
                         creation_date: {:$lte => creation_date}
                       ).order_by(creation_date: 1)
                       &.first
                       &.creation_date
      oldest_other.blank? || oldest_other > creation_date
    end
    
    # Scores update trigger
    def must_update_affiliated_scores?
      # IF
      #   any of fields list has changed
      # AND
      %w[
        creation_date
        department
        applicant_id
        is_active
      ].any? {|field_name| send("#{field_name}_changed?")} &&
        oldest_for_department?
    end
    
    # Affiliated scores are all that match
    #   SAME @department && @applicant_id
    def update_affiliated_scores
      ToucheScoresJob.perform_later(
        Hermes::Score.only(:id, :department, :applicant_id)
          .where(
            department: department,
            applicant_id: housing_request.main_applicant_id
          ).pluck(:id)
      )
    end
  end
end
